// -*- mode: typescript; coding: utf-8 -*-
// Copyright 2010, 2013, Matthias Andreas Benkard.

// -- NOTE --
// Prior to compilation, do:
//
//   git clone git://github.com/soywiz/typescript-node-definitions.git
//

///<reference path='typescript-node-definitions/node.d.ts'/>
///<reference path='typescript-node-definitions/sprintf.d.ts'/>


// Required modules.
import child_process = module("child_process");
import sprintf = module("sprintf");
var lazy = require('lazy');


// Size beautification.
var sizeSuffixes = ["B", "KiB", "MiB", "GiB", "TiB", "PiB", "EiB"];

function log1024(n: number) {
    if (n < 1024) {
        return 0;
    } else {
        return 1 + log1024(Math.floor(n / 1024));
    }
};

function beautifySize(size: number) {
    size *= 1024;
    var lognum = log1024(size);
    return sprintf.sprintf("%6.1f %3s", size / Math.pow(1024, lognum), sizeSuffixes[lognum]);
};


// Main program.
class Record {
    constructor(size: number, name: string) { this.size = size; this.name = name; }
    size: number;
    name: string;
};

function main() {
    var args = ["-k"];
    if (process.argv[2]) { args = args.concat([process.argv[2]]); }
    var records = lazy(child_process.spawn("du", args).stdout).lines.map((x: string) => {
        var parts = x.toString().split(/\t/);
        return new Record(parseInt(parts[0], 10), parts[1]);
    }).join((records: Record[]) => {
        records.sort((x: Record, y: Record) => {
            return (x.size !== y.size) ? x.size - y.size : (x.name === y.name) ? 0 : (x.name < y.name) ? 1 : -1;
        });
        records.forEach((x: Record) => {
            if (x.size) {
                console.log(sprintf.sprintf("%s\t%s", beautifySize(x.size), x.name));
            }
        });
    });
}

main();
